/*
  LinEAK - Linux support for Easy Access and Internet Keyboards      
  Copyright (C) 2001,2002 Mark Smulders <Mark@PIRnet.nl>
  
  lineakconfig, GTK+ interface for configuring lineakd

  This program is free software; you can redistribute it and/or modify  
  it under the terms of the GNU General Public License as published by  
  the Free Software Foundation; either version 2, or (at your option)   
  any later version.                                                    
                                                                        
  This program is distributed in the hope that it will be useful,       
  but WITHOUT ANY WARRANTY; without even the implied warranty of        
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
  GNU General Public License for more details.                          
                                                                        
  You should have received a copy of the GNU General Public License     
  along with this program; if not, write to the                         
  Free Software Foundation, Inc., 59 Temple Place - Suite 330,          
  Boston, MA 02111-1307, USA.                                           
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include <signal.h>

#include "callbacksproto.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"

/**********************************************************************************************/

/* if an other keyboard type is selected, update it's data on the GUI (don't apply or save yet though) */
void on_kbmenuitem_activate (GtkWidget *widget, gpointer data) {
  if (strcmp(myEAK.EAKtype,(char *)data) != 0) {
    select_new_keyboard(widget, data);
  }
}

/* another special action selected, check the radiobutton, and set the data */
void on_specmenuitem_activate (GtkWidget *widget, gpointer data) {
  GtkWidget *specialradio;
  specialradio = lookup_widget(widget, "radiospecial");
  if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(specialradio)))
	return;
  change_selectedkey_command(data);
}

/* when command radiobutton is toggled */
void on_commandradio_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
  GtkWidget *commandentry;
  gchar *entrytext;
  if (gtk_toggle_button_get_active(togglebutton)) {
	commandentry = lookup_widget(GTK_WIDGET (togglebutton), "commandentry");
	gtk_entry_set_editable(GTK_ENTRY(commandentry), TRUE);
    entrytext = gtk_editable_get_chars(GTK_EDITABLE (commandentry), 0, -1);
	change_selectedkey_command(entrytext);
    g_free(entrytext); 
  }
}

/* when action radiobutton is toggled */
void on_actionradio_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
  GtkWidget *commandentry, *actoptmenu, *actmenu, *actitem;
  if (gtk_toggle_button_get_active(togglebutton)) {
	commandentry = lookup_widget(GTK_WIDGET (togglebutton), "commandentry");
    gtk_entry_set_editable(GTK_ENTRY(commandentry), FALSE);
	actoptmenu = lookup_widget(GTK_WIDGET(togglebutton), "specials");
	actmenu = gtk_option_menu_get_menu(GTK_OPTION_MENU(actoptmenu));
	actitem = gtk_menu_get_active(GTK_MENU(actmenu));
	gtk_menu_item_activate(GTK_MENU_ITEM(actitem));
  }
}

/* user changed content of command entry */
void on_commandentry_changed (GtkEditable *editable, gpointer user_data) {
  gchar *entrytext;
  if (changing_text_from_signal)
	return;
  entrytext = gtk_editable_get_chars(editable,0,-1);
  change_selectedkey_command(entrytext);
}

/* user changed content of cdrom entry */
void on_cdromentry_changed (GtkEditable *editable, gpointer user_data) {
  gchar *entrytext;
  entrytext = gtk_editable_get_chars(editable,0,-1);
  if (strcmp(cdromdev,(char *)entrytext) != 0) {
	free(cdromdev);
	cdromdev = malloc( (strlen(entrytext)+1) *sizeof(char));
	strcpy(cdromdev,entrytext);
	strcat(cdromdev,"\0");
  }
}

/* don't allow deselection, we don't like an empty selection ;) */
void on_keylist_unselect (GtkList *list, GtkWidget *widget, gpointer user_data) {
  gtk_list_select_child(list, widget);
}

/* update GUI info on the selected key */
void on_keylist_select (GtkWidget *widget, gpointer data) {
  gint index = -1;
  if ((index = gtk_list_child_position (GTK_LIST (widget->parent),widget)) != -1 ) {
	changing_text_from_signal = TRUE;
	refresh_keycommand (widget, index);
	changing_text_from_signal = FALSE;
    selected_key = index;
  }
}

/* OK clicked */
void on_okbutton_clicked (GtkButton *button, gpointer user_data) {
  if (on_applybutton_clicked(NULL, NULL))
    cleanexit();
}

/* Apply clicked */
gboolean on_applybutton_clicked (GtkButton *button, gpointer user_data) {
  extern gboolean ApplyUsed;
  GtkWidget *warn;
  gint thepid=0;
  /* get lineakd PID */
  if ((thepid = get_lineakd_pid())==0) {
	warn = create_warning_window("ERROR: could not find lineakd running!\n"
	                             "LinEAK configuration not changed.");
	gtk_widget_show(warn);
	return FALSE;
  }
  /* first time apply? -> backup conf file */
  if (!ApplyUsed) {
	ApplyUsed = TRUE;
    backup_conffile();
  }
  /* save the changes to a new conf file */
  save_conffile();
  /* rehash lineakd */
  kill(thepid, SIGHUP);
  return TRUE;
}

/* Cancel clicked */
void on_cancelbutton_clicked (GtkButton *button, gpointer user_data) {
  extern gboolean ApplyUsed;
  gint thepid=0;
  if (ApplyUsed) {
    if ((thepid = get_lineakd_pid())==0)
	  return;
	else {
  	  restore_conffile();
      kill(thepid, SIGHUP);
	}
  }
  cleanexit();
}

/* close the alert box */
void on_alert_closebutton_clicked (GtkButton *button, gpointer user_data) {
  GtkWidget *alertwindow;
  alertwindow = lookup_widget(GTK_WIDGET (button), "warning_window");
  gtk_widget_destroy(GTK_WIDGET(alertwindow));
}

/**********************************************************************************************/

