/*
  LinEAK - Linux support for Easy Access and Internet Keyboards      
  Copyright (C) 2001,2002 Mark Smulders <Mark@PIRnet.nl>
  
  lineakconfig, GTK+ interface for configuring lineakd

  This program is free software; you can redistribute it and/or modify  
  it under the terms of the GNU General Public License as published by  
  the Free Software Foundation; either version 2, or (at your option)   
  any later version.                                                    
                                                                        
  This program is distributed in the hope that it will be useful,       
  but WITHOUT ANY WARRANTY; without even the implied warranty of        
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
  GNU General Public License for more details.                          
                                                                        
  You should have received a copy of the GNU General Public License     
  along with this program; if not, write to the                         
  Free Software Foundation, Inc., 59 Temple Place - Suite 330,          
  Boston, MA 02111-1307, USA.                                           
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "callbacksproto.h"
#include "interface.h"
#include "support.h"

#include "pixmaps.h"

/**********************************************************************************************/

/* create the main window */
GtkWidget* create_MainWindow (void) {
  GtkWidget *MainWindow;
  GtkWidget *fixed1;
  GtkWidget *pixmaplogo;
  GtkWidget *frame1;
  GtkWidget *fixed2;
  GtkWidget *cdromentry;
  GtkWidget *kbtypes;
  GtkWidget *kbtypes_menu;
  GtkWidget *label1;
  GtkWidget *label2;
  GtkWidget *kbimage;
  GtkWidget *frame2;
  GtkWidget *fixed3;
  GtkWidget *commandentry;
  GtkWidget *specials;
  GtkWidget *specials_menu;
  GtkWidget *scrolledwindow1;
  GtkWidget *viewport1;
  GtkWidget *keylist;
  GSList *KeyActionGroup_group = NULL;
  GtkWidget *radiocommand;
  GtkWidget *radiospecial;
  GtkWidget *label3;
  GtkWidget *frame4;
  GtkWidget *fixed5;
  GtkWidget *button10;
  GtkWidget *hbox4;
  GtkWidget *label18;
  GtkWidget *pixmap4;
  GtkWidget *label19;
  GtkWidget *label20;
  GtkWidget *button11;
  GtkWidget *hbox5;
  GtkWidget *label21;
  GtkWidget *pixmap5;
  GtkWidget *label22;
  GtkWidget *label23;
  GtkWidget *button12;
  GtkWidget *hbox6;
  GtkWidget *label24;
  GtkWidget *pixmap6;
  GtkWidget *label25;
  GtkWidget *label26;

  MainWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (MainWindow), "MainWindow", MainWindow);
  gtk_container_set_border_width (GTK_CONTAINER (MainWindow), 2);
  gtk_window_set_title (GTK_WINDOW (MainWindow), _("LinEAK Configuration"));
  gtk_window_set_position (GTK_WINDOW (MainWindow), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW (MainWindow), 300, 600);
  gtk_window_set_policy (GTK_WINDOW (MainWindow), FALSE, FALSE, FALSE);
  gtk_signal_connect (GTK_OBJECT (MainWindow), "delete-event", GTK_SIGNAL_FUNC (on_cancelbutton_clicked), NULL);

  fixed1 = gtk_fixed_new ();
  gtk_widget_ref (fixed1);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "fixed1", fixed1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fixed1);
  gtk_container_add (GTK_CONTAINER (MainWindow), fixed1);

/* lineak logo */
  //pixmaplogo = create_my_pixmap (MainWindow, lineak_logo_xpm);
  pixmaplogo = create_my_pixmap_from_file (MainWindow, "lineak_logo.xpm");
  gtk_widget_ref (pixmaplogo);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "pixmaplogo", pixmaplogo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pixmaplogo);
  gtk_fixed_put (GTK_FIXED (fixed1), pixmaplogo, 4, 4);

/* global settings */
  frame1 = gtk_frame_new (_("Global settings"));
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_fixed_put (GTK_FIXED (fixed1), frame1, 0, 100);
  gtk_widget_set_uposition (frame1, 0, 100);
  gtk_widget_set_usize (frame1, 432, 187);

  fixed2 = gtk_fixed_new ();
  gtk_widget_ref (fixed2);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "fixed2", fixed2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fixed2);
  gtk_container_add (GTK_CONTAINER (frame1), fixed2);

  cdromentry = gtk_entry_new ();
  gtk_widget_ref (cdromentry);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "cdromentry", cdromentry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (cdromentry), "changed", GTK_SIGNAL_FUNC (on_cdromentry_changed), NULL);
  gtk_widget_show (cdromentry);
  gtk_fixed_put (GTK_FIXED (fixed2), cdromentry, 16, 96);
  gtk_widget_set_uposition (cdromentry, 16, 96);
  gtk_widget_set_usize (cdromentry, 158, 22);

  kbtypes = gtk_option_menu_new ();
  gtk_widget_ref (kbtypes);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "kbtypes", kbtypes,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (kbtypes);
  gtk_fixed_put (GTK_FIXED (fixed2), kbtypes, 16, 32);
  gtk_widget_set_uposition (kbtypes, 16, 32);
  gtk_widget_set_usize (kbtypes, 392, 24);
  kbtypes_menu = gtk_menu_new ();
  gtk_option_menu_set_menu (GTK_OPTION_MENU (kbtypes), kbtypes_menu);

  label1 = gtk_label_new (_("Keyboard type:"));
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_fixed_put (GTK_FIXED (fixed2), label1, 16, 8);
  gtk_widget_set_uposition (label1, 16, 8);
  gtk_widget_set_usize (label1, 88, 24);

  label2 = gtk_label_new (_("CD-ROM device:"));
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_fixed_put (GTK_FIXED (fixed2), label2, 16, 72);
  gtk_widget_set_uposition (label2, 16, 72);
  gtk_widget_set_usize (label2, 96, 24);
  
  /* keyboard image... dynamically created at runtime */
  kbimage = create_my_pixmap (MainWindow, dummy_pixmap_xpm);
  gtk_widget_ref (kbimage);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "kbimage", kbimage,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (kbimage);
  gtk_fixed_put (GTK_FIXED (fixed2), kbimage, 190, 72);

/* keyboard settings */
  frame2 = gtk_frame_new (_("Keyboard settings"));
  gtk_widget_ref (frame2);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "frame2", frame2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame2);
  gtk_fixed_put (GTK_FIXED (fixed1), frame2, 0, 260);
  gtk_widget_set_uposition (frame2, 0, 295);
  gtk_widget_set_usize (frame2, 432, 240);

  fixed3 = gtk_fixed_new ();
  gtk_widget_ref (fixed3);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "fixed3", fixed3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fixed3);
  gtk_container_add (GTK_CONTAINER (frame2), fixed3);

  commandentry = gtk_entry_new ();
  gtk_widget_ref (commandentry);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "commandentry", commandentry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (commandentry), "changed", GTK_SIGNAL_FUNC (on_commandentry_changed), NULL);
  gtk_widget_show (commandentry);
  gtk_fixed_put (GTK_FIXED (fixed3), commandentry, 208, 64);
  gtk_widget_set_uposition (commandentry, 208, 64);
  gtk_widget_set_usize (commandentry, 200, 24);

  specials = gtk_option_menu_new ();
  gtk_widget_ref (specials);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "specials", specials,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (specials);
  gtk_fixed_put (GTK_FIXED (fixed3), specials, 208, 136);
  gtk_widget_set_uposition (specials, 208, 136);
  gtk_widget_set_usize (specials, 200, 24);
  specials_menu = gtk_menu_new ();
  gtk_option_menu_set_menu (GTK_OPTION_MENU (specials), specials_menu);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow1);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "scrolledwindow1", scrolledwindow1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow1);
  gtk_fixed_put (GTK_FIXED (fixed3), scrolledwindow1, 16, 32);
  gtk_widget_set_uposition (scrolledwindow1, 16, 32);
  gtk_widget_set_usize (scrolledwindow1, 152, 176);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

  viewport1 = gtk_viewport_new (NULL, NULL);
  gtk_widget_ref (viewport1);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "viewport1", viewport1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (viewport1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), viewport1);

  keylist = gtk_list_new ();
  gtk_widget_ref (keylist);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "keylist", keylist,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_list_set_selection_mode(GTK_LIST (keylist), GTK_SELECTION_SINGLE);
  gtk_widget_show (keylist);
  gtk_container_add (GTK_CONTAINER (viewport1), keylist);

  radiocommand = gtk_radio_button_new_with_label (KeyActionGroup_group, _("Run command:"));
  KeyActionGroup_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiocommand));
  gtk_widget_ref (radiocommand);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "radiocommand", radiocommand,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (radiocommand), "toggled", GTK_SIGNAL_FUNC (on_commandradio_toggled), NULL);
//  gtk_signal_connect (GTK_OBJECT (radiocommand), "focus-out-event", GTK_SIGNAL_FUNC (on_radiocommand_focusout), NULL);
  gtk_widget_show (radiocommand);
  gtk_fixed_put (GTK_FIXED (fixed3), radiocommand, 192, 40);
  gtk_widget_set_uposition (radiocommand, 192, 40);
  gtk_widget_set_usize (radiocommand, 104, 24);

  radiospecial = gtk_radio_button_new_with_label (KeyActionGroup_group, _("Perform special action:"));
  KeyActionGroup_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiospecial));
  gtk_widget_ref (radiospecial);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "radiospecial", radiospecial,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (radiospecial), "toggled", GTK_SIGNAL_FUNC (on_actionradio_toggled), NULL);
//  gtk_signal_connect (GTK_OBJECT (radiospecial), "focus-out-event", GTK_SIGNAL_FUNC (on_radiospecial_focusout), NULL);
  gtk_widget_show (radiospecial);
  gtk_fixed_put (GTK_FIXED (fixed3), radiospecial, 192, 112);
  gtk_widget_set_uposition (radiospecial, 192, 112);
  gtk_widget_set_usize (radiospecial, 152, 24);

  label3 = gtk_label_new (_("Available keys:"));
  gtk_widget_ref (label3);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "label3", label3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label3);
  gtk_fixed_put (GTK_FIXED (fixed3), label3, 16, 8);
  gtk_widget_set_uposition (label3, 16, 8);
  gtk_widget_set_usize (label3, 88, 24);

/* buttons */
  frame4 = gtk_frame_new (NULL);
  gtk_widget_ref (frame4);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "frame4", frame4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame4);
  gtk_fixed_put (GTK_FIXED (fixed1), frame4, 0, 543);
  gtk_widget_set_uposition (frame4, 0, 543);
  gtk_widget_set_usize (frame4, 432, 48);
  gtk_frame_set_shadow_type (GTK_FRAME (frame4), GTK_SHADOW_NONE);

  fixed5 = gtk_fixed_new ();
  gtk_widget_ref (fixed5);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "fixed5", fixed5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fixed5);
  gtk_container_add (GTK_CONTAINER (frame4), fixed5);

  /* OK */
  button10 = gtk_button_new ();
  gtk_widget_ref (button10);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "button10", button10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button10), "clicked", GTK_SIGNAL_FUNC (on_okbutton_clicked), NULL);
  gtk_widget_show (button10);
  gtk_fixed_put (GTK_FIXED (fixed5), button10, 128, 0);
  gtk_widget_set_uposition (button10, 128, 0);
  gtk_widget_set_usize (button10, 88, 40);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox4);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "hbox4", hbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox4);
  gtk_container_add (GTK_CONTAINER (button10), hbox4);

  label18 = gtk_label_new ("    ");
  gtk_widget_ref (label18);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "label18", label18,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label18);
  gtk_box_pack_start (GTK_BOX (hbox4), label18, FALSE, FALSE, 0);

  pixmap4 = create_my_pixmap (MainWindow, ok_pixmap_xpm);
  gtk_widget_ref (pixmap4);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "pixmap4", pixmap4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pixmap4);
  gtk_box_pack_start (GTK_BOX (hbox4), pixmap4, TRUE, TRUE, 0);

  label19 = gtk_label_new (_("OK"));
  gtk_widget_ref (label19);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "label19", label19,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label19);
  gtk_box_pack_start (GTK_BOX (hbox4), label19, FALSE, FALSE, 0);

  label20 = gtk_label_new ("     ");
  gtk_widget_ref (label20);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "label20", label20,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label20);
  gtk_box_pack_start (GTK_BOX (hbox4), label20, FALSE, FALSE, 0);

  /* Apply */
  button11 = gtk_button_new ();
  gtk_widget_ref (button11);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "button11", button11,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button11), "clicked", GTK_SIGNAL_FUNC (on_applybutton_clicked), NULL);
  gtk_widget_show (button11);
  gtk_fixed_put (GTK_FIXED (fixed5), button11, 232, 0);
  gtk_widget_set_uposition (button11, 232, 0);
  gtk_widget_set_usize (button11, 88, 40);

  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox5);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "hbox5", hbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox5);
  gtk_container_add (GTK_CONTAINER (button11), hbox5);

  label21 = gtk_label_new ("  ");
  gtk_widget_ref (label21);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "label21", label21,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label21);
  gtk_box_pack_start (GTK_BOX (hbox5), label21, FALSE, FALSE, 0);

  pixmap5 = create_my_pixmap (MainWindow, apply_pixmap_xpm);
  gtk_widget_ref (pixmap5);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "pixmap5", pixmap5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pixmap5);
  gtk_box_pack_start (GTK_BOX (hbox5), pixmap5, TRUE, TRUE, 0);

  label22 = gtk_label_new (_("Apply"));
  gtk_widget_ref (label22);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "label22", label22,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label22);
  gtk_box_pack_start (GTK_BOX (hbox5), label22, FALSE, FALSE, 0);

  label23 = gtk_label_new ("    ");
  gtk_widget_ref (label23);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "label23", label23,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label23);
  gtk_box_pack_start (GTK_BOX (hbox5), label23, FALSE, FALSE, 0);

  /* Cancel */
  button12 = gtk_button_new ();
  gtk_widget_ref (button12);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "button12", button12,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button12), "clicked", GTK_SIGNAL_FUNC (on_cancelbutton_clicked), NULL);
  gtk_widget_show (button12);
  gtk_fixed_put (GTK_FIXED (fixed5), button12, 336, 0);
  gtk_widget_set_uposition (button12, 336, 0);
  gtk_widget_set_usize (button12, 88, 40);

  hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox6);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "hbox6", hbox6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox6);
  gtk_container_add (GTK_CONTAINER (button12), hbox6);

  label24 = gtk_label_new ("  ");
  gtk_widget_ref (label24);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "label24", label24,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label24);
  gtk_box_pack_start (GTK_BOX (hbox6), label24, FALSE, FALSE, 0);

  pixmap6 = create_my_pixmap (MainWindow, cancel_pixmap_xpm);
  gtk_widget_ref (pixmap6);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "pixmap6", pixmap6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pixmap6);
  gtk_box_pack_start (GTK_BOX (hbox6), pixmap6, TRUE, TRUE, 0);

  label25 = gtk_label_new (_("Cancel"));
  gtk_widget_ref (label25);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "label25", label25,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label25);
  gtk_box_pack_start (GTK_BOX (hbox6), label25, FALSE, FALSE, 0);

  label26 = gtk_label_new ("   ");
  gtk_widget_ref (label26);
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "label26", label26,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label26);
  gtk_box_pack_start (GTK_BOX (hbox6), label26, FALSE, FALSE, 0);

  return MainWindow;
}

/**********************************************************************************************/

GtkWidget *create_warning_window (gchar *warntext) {
  GtkWidget *warning_window;
  GtkWidget *fixed1;
  GtkWidget *pixmap1;
  GtkWidget *error_label;
  GtkWidget *alert_close_button;

  warning_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (warning_window), "warning_window", warning_window);
  gtk_window_set_title (GTK_WINDOW (warning_window), "LinEAK ALERT");
  gtk_container_set_border_width (GTK_CONTAINER (warning_window), 5);
  gtk_window_set_position (GTK_WINDOW (warning_window), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (warning_window), TRUE);
  gtk_window_set_policy (GTK_WINDOW (warning_window), FALSE, FALSE, FALSE);

  fixed1 = gtk_fixed_new ();
  gtk_widget_ref (fixed1);
  gtk_object_set_data_full (GTK_OBJECT (warning_window), "fixed1", fixed1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fixed1);
  gtk_container_add (GTK_CONTAINER (warning_window), fixed1);

  //pixmap1 = create_pixmap (warning_window, NULL);
  pixmap1 = create_my_pixmap_from_file (warning_window, "warning.xpm");
  gtk_widget_ref (pixmap1);
  gtk_object_set_data_full (GTK_OBJECT (warning_window), "pixmap1", pixmap1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pixmap1);
  gtk_fixed_put (GTK_FIXED (fixed1), pixmap1, 0, 0);
  //gtk_widget_set_uposition (pixmap1, 16, 16);
  //gtk_widget_set_usize (pixmap1, 21, 21);

  alert_close_button = gtk_button_new_with_label ("Close");
  gtk_widget_ref (alert_close_button);
  gtk_object_set_data_full (GTK_OBJECT (warning_window), "alert_close_button", alert_close_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (alert_close_button), "clicked", GTK_SIGNAL_FUNC (on_alert_closebutton_clicked), NULL);
  gtk_widget_show (alert_close_button);
  gtk_fixed_put (GTK_FIXED (fixed1), alert_close_button, 112, 80);
  gtk_widget_set_uposition (alert_close_button, 112, 80);
  gtk_widget_set_usize (alert_close_button, 72, 30);

  error_label = gtk_label_new (warntext);
  gtk_widget_ref (error_label);
  gtk_object_set_data_full (GTK_OBJECT (warning_window), "error_label", error_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (error_label);
  gtk_fixed_put (GTK_FIXED (fixed1), error_label, 56, 0);
  gtk_widget_set_uposition (error_label, 56, 0);
  gtk_widget_set_usize (error_label, 240, 80);
  gtk_label_set_justify (GTK_LABEL (error_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (error_label), TRUE);

  return warning_window;
}

/**********************************************************************************************/
