/*
  LinEAK - Linux support for Easy Access and Internet Keyboards      
  Copyright (C) 2001,2002 Mark Smulders <Mark@PIRnet.nl>
  
  lineakconfig, GTK+ interface for configuring lineakd

  This program is free software; you can redistribute it and/or modify  
  it under the terms of the GNU General Public License as published by  
  the Free Software Foundation; either version 2, or (at your option)   
  any later version.                                                    
                                                                        
  This program is distributed in the hope that it will be useful,       
  but WITHOUT ANY WARRANTY; without even the implied warranty of        
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
  GNU General Public License for more details.                          
                                                                        
  You should have received a copy of the GNU General Public License     
  along with this program; if not, write to the                         
  Free Software Foundation, Inc., 59 Temple Place - Suite 330,          
  Boston, MA 02111-1307, USA.                                           
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <gtk/gtk.h>

/* global stuff */
#include "support.h"
#include "callbacksproto.h"

/* private stuff */
#include "supportpriv.h"
#include "lineakconfig.h"


/**********************************************************************************************/

/* refresh *ALL* the data on the GUI (this is called on startup) */
gint refreshGUIdata(GtkWidget *widget) {
  extern gboolean changing_text_from_signal;
  
  changing_text_from_signal = TRUE;
  /* refresh EAK types */
  refresh_kbtypes (widget);
  
  /* refresh CD-ROM entry */
  refresh_cdromdev (widget);
  
  /* refresh keyboard image */
  refresh_kbimage (widget);

  /* refresh key list */
  refresh_keylist (widget);
  
  /* refresh command/action */
  refresh_specials (widget);
  refresh_keycommand (widget, 0);
  changing_text_from_signal = FALSE;
  
  return (TRUE);
}

/* only needed at startup */
gint refresh_kbtypes (GtkWidget *widget) {
  GtkWidget *guiwidget, *menuwidget;
  gint i=0;
  guiwidget = lookup_widget(widget, "kbtypes");
  gtk_option_menu_remove_menu(GTK_OPTION_MENU (guiwidget));
  menuwidget = gtk_menu_new();
  for (i=0; i<nrkeyboards; i++)
    add_kbmenu_item(menuwidget, i, longkbname[i], TRUE);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (guiwidget), menuwidget);
  gtk_option_menu_set_history(GTK_OPTION_MENU (guiwidget), (guint)myEAK.EAKnr);
  return (TRUE);
}

/* only needed at startup */
gint refresh_cdromdev (GtkWidget *widget) {
  GtkWidget *guiwidget;
  guiwidget = lookup_widget(widget, "cdromentry");
  gtk_entry_set_text(GTK_ENTRY (guiwidget), (gchar *)cdromdev);
  return (TRUE);  
}

gint refresh_kbimage (GtkWidget *widget) {
  GtkWidget *kbimage;
  char *pixmapfile;
  kbimage = lookup_widget(widget, "kbimage");
  gtk_widget_hide(kbimage);
  pixmapfile = malloc( (strlen(myEAK.EAKtype) + 5) * sizeof(char));
  strcpy(pixmapfile, myEAK.EAKtype);
  strcat(pixmapfile, ".xpm\0");
  replace_my_pixmap_from_file(widget, kbimage,pixmapfile);
  gtk_widget_show(kbimage);
  free(pixmapfile);
  return (TRUE);
}

/* refresh the keylist when the user selected some other keyboard */
gint refresh_keylist (GtkWidget *widget) {
  GtkWidget *guiwidget;
  EAkey *thiskey=NULL;
  guiwidget = lookup_widget(widget, "keylist");
  gtk_list_clear_items(GTK_LIST (guiwidget), 0, -1);
  for (thiskey=myEAK.EAKeylist; thiskey!=NULL; thiskey=thiskey->next)
	add_list_item(guiwidget, thiskey->EAkeyname);
  gtk_signal_connect (GTK_OBJECT (guiwidget), "unselect-child", GTK_SIGNAL_FUNC (on_keylist_unselect), NULL);
  gtk_list_select_item(GTK_LIST (guiwidget), 0);
  selected_key = 0;
  return (TRUE);  
}

/* refresh the command bound to a key when it's selected */
gint refresh_keycommand (GtkWidget *widget, gint keyindex) {
  extern gboolean changing_text_from_signal;
  GtkWidget *guiwidget;
  guint i=0;
  EAkey *thiskey;
  changing_text_from_signal = TRUE;
  for (i=0, thiskey=myEAK.EAKeylist; i!=keyindex && thiskey!=NULL; i++, thiskey=thiskey->next);
  if (thiskey==NULL) {
	g_warning("invalid key index: %d\n",keyindex);
	return (FALSE);
  }
  if (is_special_action((gchar *)(thiskey->EAcommand))) { /* special action */
	guiwidget = lookup_widget(widget, "radiocommand");
//	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (guiwidget), FALSE);
	gtk_widget_hide(guiwidget);
	GTK_TOGGLE_BUTTON(guiwidget)->active = FALSE;
	gtk_widget_show(guiwidget);
	guiwidget = lookup_widget(widget, "radiospecial");
//	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (guiwidget), TRUE);
	gtk_widget_hide(guiwidget);
	GTK_TOGGLE_BUTTON(guiwidget)->active = TRUE;
	gtk_widget_show(guiwidget);
	guiwidget = lookup_widget(widget, "commandentry");
	gtk_entry_set_text(GTK_ENTRY (guiwidget), "");
	gtk_entry_set_editable(GTK_ENTRY(guiwidget), FALSE);
	for (i=0; strcmp(specialacts[i].action,thiskey->EAcommand)!=0 && i<NR_SPECIALS; i++);
    guiwidget = lookup_widget(widget, "specials");
	gtk_option_menu_set_history(GTK_OPTION_MENU (guiwidget), i);
  } else {                                                        /* system command */
	guiwidget = lookup_widget(widget, "radiospecial");
//	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (guiwidget), FALSE);
	gtk_widget_hide(guiwidget);
	GTK_TOGGLE_BUTTON(guiwidget)->active = FALSE;
	gtk_widget_show(guiwidget);
	guiwidget = lookup_widget(widget, "radiocommand");
//	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (guiwidget), TRUE);
	gtk_widget_hide(guiwidget);
	GTK_TOGGLE_BUTTON(guiwidget)->active = TRUE;
	gtk_widget_show(guiwidget);
	guiwidget = lookup_widget(widget, "commandentry");
	gtk_entry_set_text(GTK_ENTRY (guiwidget), (gchar *)(thiskey->EAcommand));
	gtk_entry_set_editable(GTK_ENTRY(guiwidget), TRUE);
  }
  changing_text_from_signal = FALSE;
  return (TRUE);  
}

/* refill special action menu (only needed on startup) */
gint refresh_specials (GtkWidget *widget) {
  GtkWidget *guiwidget, *menuwidget;
  gint i=0;
  guiwidget = lookup_widget(widget, "specials");
  gtk_option_menu_remove_menu(GTK_OPTION_MENU (guiwidget));
  menuwidget = gtk_menu_new();
  for (i=0; i<NR_SPECIALS; i++)
    add_specmenu_item(menuwidget, specialacts[i].action, specialacts[i].longname, TRUE);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (guiwidget), menuwidget);
  return (TRUE);
}

/* convenient for adding new menu items */
GtkWidget *add_kbmenu_item (GtkWidget *menu, gint index, gchar *name, gint add_callback) {
  GtkWidget *item;
  item = gtk_menu_item_new_with_label (name);
  gtk_menu_append (GTK_MENU (menu), item);
  if (add_callback)
	gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC (on_kbmenuitem_activate), cfgSectionNumberToName(index));
  gtk_widget_show (item);
  return item;
}

/* convenient for adding new menu items */
GtkWidget *add_specmenu_item (GtkWidget *menu, char *act, gchar *name, gint add_callback) {
  GtkWidget *item;
  item = gtk_menu_item_new_with_label (name);
  gtk_menu_append (GTK_MENU (menu), item);
  if (add_callback)
	gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC (on_specmenuitem_activate), act);
  gtk_widget_show (item);
  return item;
}

/* convenient for adding new list items */
GtkWidget *add_list_item (GtkWidget *list, gchar *name) {
  GtkWidget *item;
  item = gtk_list_item_new_with_label (name);
  gtk_container_add(GTK_CONTAINER (list), item);
  gtk_signal_connect (GTK_OBJECT (item), "select", GTK_SIGNAL_FUNC (on_keylist_select), name);
  gtk_widget_show (item);
  return item;
}

/* is a command a special action? */
gint is_special_action (gchar *command) {
  if (strcmp(command, EAK_EJECT)==0 ||
	  strcmp(command, EAK_VOLUP)==0 ||
      strcmp(command, EAK_VOLDOWN)==0 ||
      strcmp(command, EAK_MUTE)==0 ||
      strcmp(command, EAK_SLEEP)==0)
    return (TRUE);
  else
	return (FALSE);
}

/* select a new keyboard */
void select_new_keyboard(GtkWidget *widget, gpointer data) {
  extern gboolean changing_text_from_signal;
  free(kbtype);
  kbtype=malloc(strlen(data)*sizeof(char));
  strcpy(kbtype, (char *)data);
  initEAK();
  changing_text_from_signal = TRUE;
  refresh_keylist (widget);
  refresh_keycommand (widget, 0); /* refresh_keylist selects 1st key, hence index 0 */
  refresh_kbimage (widget);
  changing_text_from_signal = FALSE;
  selected_key = 0;
}

/* change the key command data to the users input */
void change_key_command(EAkey *thiskey, gchar *entrytext) {
  free(thiskey->EAcommand);
  thiskey->EAcommand = malloc((strlen(entrytext)+1)*sizeof(char));
  strcpy(thiskey->EAcommand,entrytext);
  strcat(thiskey->EAcommand,"\0");
}

/* wrapper for change_key_command, lookups the selected key */
void change_selectedkey_command(gchar *entrytext) {
  EAkey *thiskey;
  guint i;
  for (i=0, thiskey=myEAK.EAKeylist; i!=selected_key && thiskey!=NULL; i++, thiskey=thiskey->next);
  if (thiskey==NULL) {
    g_warning("invalid selected key index: %d\n",selected_key);
    g_free(entrytext);
    return;
  }
  if (strcmp(thiskey->EAcommand,(char *)entrytext) != 0)
	change_key_command(thiskey, entrytext);
}

/**********************************************************************************************/

/* for looking up a widget */
GtkWidget *lookup_widget (GtkWidget *widget, const gchar *widget_name) {
  GtkWidget *parent, *found_widget;
  for (;;) {
    if (GTK_IS_MENU (widget))
      parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
    else
      parent = widget->parent;
    if (parent == NULL)
      break;
    widget = parent;
  }
  found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget), widget_name);
  if (!found_widget)
    g_warning ("Widget not found: %s", widget_name);
  return found_widget;
}

/* This is an internally used function to create pixmaps. */
GtkWidget* create_my_pixmap (GtkWidget *widget, char **thepixmap) {
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;

  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, colormap, &mask,
                                                     NULL, thepixmap);
  if (gdkpixmap == NULL)
    g_error ("Couldn't create pixmap.");
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return pixmap;
}

/* This is an internally used function to create pixmaps. */
GtkWidget *create_my_pixmap_from_file (GtkWidget *widget, const gchar *filename) {
  gchar *found_filename = NULL;
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;

  if (!filename || !filename[0])
      return create_my_pixmap (widget, dummy_pixmap_xpm);

  found_filename = check_file_exists (PACKAGE_PIXMAPS_DIR, filename);

  /* If we haven't found the pixmap, try the source directory (so I see a pixmap while developing ;-)) */
  if (!found_filename) {
    found_filename = check_file_exists ("../pixmaps", filename);
  }
  if (!found_filename) {
    found_filename = check_file_exists ("pixmaps", filename);
  }

  /* still not found... warning and a dummy pixmap */
  if (!found_filename) {
    g_warning (_("Couldn't find pixmap file: %s"), filename);
    return create_my_pixmap (widget, dummy_pixmap_xpm);
  }

  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask, NULL, found_filename);
  if (gdkpixmap == NULL) {
    g_warning (_("Error loading pixmap file: %s"), found_filename);
    g_free (found_filename);
    return create_my_pixmap (widget, dummy_pixmap_xpm);
  }
  g_free (found_filename);
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return pixmap;
}

void replace_my_pixmap_from_file (GtkWidget *widget, GtkWidget *thepixmap, const gchar *filename) {
  gchar *found_filename = NULL;
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap=NULL;
  GdkBitmap *mask;
  gboolean noimage = FALSE;

  if (!filename || !filename[0]) {
	noimage = TRUE;
  } else {

    found_filename = check_file_exists (PACKAGE_PIXMAPS_DIR, filename);

    /* If we haven't found the pixmap, try the source directory (so I see a pixmap while developing ;-)) */
    if (!found_filename) {
      found_filename = check_file_exists ("../pixmaps", filename);
    }
    if (!found_filename) {
      found_filename = check_file_exists ("pixmaps", filename);
    }

    /* still not found... warning and a dummy pixmap */
    if (!found_filename) {
	  noimage = TRUE;
    }
  }
 
  colormap = gtk_widget_get_colormap (widget);
  if (noimage) {
    found_filename = check_file_exists (PACKAGE_PIXMAPS_DIR, NOIMAGEXPM);
    if (!found_filename)
      found_filename = check_file_exists ("../pixmaps", NOIMAGEXPM);
    if (!found_filename)
      found_filename = check_file_exists ("pixmaps", NOIMAGEXPM);
  }
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask, NULL, found_filename);
  if (gdkpixmap == NULL) {
    g_warning (_("Error reloading pixmap file: %s"), found_filename);
    g_free (found_filename);
    return;
  }
  g_free (found_filename);
  gtk_pixmap_set (GTK_PIXMAP(thepixmap), gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return;
}

/* This is an internally used function to check if a pixmap file exists. */
gchar *check_file_exists (const gchar *directory, const gchar *filename) {
  gchar *full_filename;
  struct stat s;
  gint status;

  full_filename = (gchar*) g_malloc (strlen (directory) + 1
                                     + strlen (filename) + 1);
  strcpy (full_filename, directory);
  strcat (full_filename, G_DIR_SEPARATOR_S);
  strcat (full_filename, filename);

  status = stat (full_filename, &s);
  if (status == 0 && S_ISREG (s.st_mode))
    return full_filename;
  g_free (full_filename);
  return NULL;
}

/**********************************************************************************************/

/* parse .conf file */
gint parseconffile(void) {
  FILE *fp = NULL;
  gchar *homedir = NULL;

  if ( (homedir = (gchar *) getenv("HOME")) != NULL ) {
    cfile = g_malloc((strlen(homedir) + strlen(CONFFILE) + 1) * sizeof(gchar));
    strcpy(cfile, homedir);
    strcat(cfile, CONFFILE "\0");

    /* does the file exist? */
    if ((fp = fopen(cfile, "r")) != NULL) {
	  fclose(fp);
	  /* backup filename */
	  backupfile = g_malloc((strlen(homedir) + strlen(BAKFILE) + 1) * sizeof(gchar));
      strcpy(backupfile, homedir);
      strcat(backupfile, BAKFILE "\0");
	  /* parse the contents of the conf file */
	  if ((cfgParse(cfile, confdata, CFG_SIMPLE)) == -1) {
	    /* error parsing file */
	    g_print("FATAL ERROR occurred while reading configuration data!\n");
	    return (FALSE);
	  }
    } else {
	  /* file does not exist */
      g_print("FATAL ERROR: configuration file %s not found!\n             use 'lineakd -c TYPE' to create one\n",cfile);
  	  return (FALSE);
    }
  
  } else {
    g_print("ERROR: no $HOME environment setting found\n");
    return (FALSE);
  }
  
  return (TRUE);
}

/* parse keyboard definition file */
gint parsedeffile(void) {
  FILE *fp = NULL;

  /* SET full path of definition file ($sysconfdir/lineakkb.def) */
  dfile = malloc((strlen(CONFDIR) + 1 + strlen(DEFFILE) + 1)*sizeof(char));
  strcpy(dfile, CONFDIR);
  strcat(dfile, "/");
  strcat(dfile, DEFFILE);
  strcat(dfile, "\0");

  /* does the file exist? */
  if ((fp = fopen(dfile, "r")) != NULL) {
    fclose(fp);
	/* it exists, parse the contents */
	if ((nrkeyboards = cfgParse(dfile, defdata, CFG_INI)) == -1) {
	  /* error parsing file */
	  g_print("FATAL ERROR occurred while reading definition data!\n");
  	  return(FALSE);
	}
  } else {
	/* file does not exist */
    g_print("FATAL ERROR: definition file %s not found!\n",dfile);
	return(FALSE);
  }

  return (TRUE);
}

/**********************************************************************************************/

/* cleanup */
gint cleanexit() {
  cleanKeylist();
  gtk_main_quit();
  return FALSE;
}

/**********************************************************************************************/

/* init the EAKeyboard */
gint initEAK (void) {
  cfgList *l = NULL;
  EAkey *tmp = NULL;
  gint keycount = 0;
  cleanKeylist();
  myEAK.EAKnr = cfgSectionNameToNumber(kbtype);
  myEAK.EAKtype = kbtype;
  myEAK.EAKname = longkbname[myEAK.EAKnr];
  if (keylist[myEAK.EAKnr] != NULL) {
	myEAK.EAKeylist = tmp = (EAkey *) g_malloc(sizeof(EAkey));
	for (l=keylist[myEAK.EAKnr]; l!=NULL; l=l->next) {
      keycount++;
	  tmp->EAkeyname = l->str;

	  //if (fetchVarFromCfgFile(dfile, tmp->EAkeyname, &(tmp->EAkeycode), CFG_INT, CFG_INI, 0, kbtype) == -1) {
		//g_print("FATAL ERROR while fetching keycode for key '%s' from definition file!\n",tmp->EAkeyname);
	    //return (FALSE); 
	  //}
	  //tmp->EAkeysym = 0x1008FF01 + keycount;
	  tmp->EAcommand = NULL;
	  if (fetchVarFromCfgFile(cfile, tmp->EAkeyname, &(tmp->EAcommand), CFG_STRING, CFG_SIMPLE, 0, NULL) == -1) {
		free(tmp->EAcommand);
		tmp->EAcommand = g_malloc(1*sizeof(char));
	    strcpy(tmp->EAcommand,"\0");
		//g_print("FATAL ERROR while fetching command for key '%s' from definition file!\n",tmp->EAkeyname);
	    //return (FALSE); 
	  } 
	  if (l->next != NULL) {
		tmp->next = (EAkey *) g_malloc(sizeof(EAkey));
		tmp = tmp->next;
	  } else {
		tmp->next = NULL;
	  }
	}
  } else {
	myEAK.EAKeylist = NULL;
  }
  return (TRUE);
}

/* destroy the EAkeys linked list */
void cleanKeylist (void) {
  EAkey *tmp = NULL, *tmp2 = NULL;
  tmp = myEAK.EAKeylist;
  while (tmp!=NULL) {
	tmp2 = tmp->next;
	free(tmp->EAcommand);
	free(tmp);
	tmp = tmp2;
  }
}

/**********************************************************************************************/

/* backup the conf file so we can cancel the changes */
gboolean backup_conffile (void) {
  FILE *orig, *backup;
  int chr;
  if ((orig=fopen(cfile,"r"))==NULL) {
	g_warning("error opening configuration file %s for read\n",cfile);
	return FALSE;
  }
  if ((backup=fopen(backupfile, "w"))==NULL) {
	g_warning("error opening backup configuration file %s for write\n",backupfile);
	fclose(orig);
	return FALSE;
  }
  while ((chr=fgetc(orig))!=EOF) fputc(chr,backup);
  fclose(orig);
  fclose(backup);
  return TRUE;
}

/* restore the conf file from backup */
gboolean restore_conffile (void) {
  FILE *orig, *dest;
  int chr;
  if ((orig=fopen(backupfile,"r"))==NULL) {
	g_warning("error opening backup configuration file %s for read\n",backupfile);
	return FALSE;
  }
  if ((dest=fopen(cfile, "w"))==NULL) {
	g_warning("error opening configuration file %s for write\n",cfile);
	fclose(orig);
	return FALSE;
  }
  while ((chr=fgetc(orig))!=EOF) fputc(chr,dest);
  fclose(orig);
  fclose(dest);
  return TRUE;
}

/* save the settings to the conf file */
gboolean save_conffile (void) {
  FILE *dest;
  EAkey *key;
  if ((dest=fopen(cfile, "w"))==NULL) {
	g_warning("error opening configuration file %s for write\n",cfile);
	return FALSE;
  }
  fprintf(dest,"%s# LinEAK Configuration for %s\n# (generated by lineakconfig)\n\n# Global settings\n"
               "KeyboardType\t\t= %s\n"
               "CdromDevice\t\t= %s\n"
               "\n# Specific keys of your keyboard\n", CONF_HEADER, myEAK.EAKname, kbtype, cdromdev);
  key = myEAK.EAKeylist;
  while (key!=NULL) {
	fprintf(dest,"%s%s= \"%s\"\n",key->EAkeyname,strlen(key->EAkeyname)>7?"\t":"\t\t",key->EAcommand);
	key = key->next;
  }
  fprintf(dest,"\n# end lineakd.conf\n");
  fclose(dest);
  return TRUE;
}

gint get_lineakd_pid(void) {
  FILE *fp;
  char *home, *pidfile;
  gint thepid=0;
  home = getenv("HOME");
  pidfile = malloc( (strlen(home)+strlen(PIDFILE)+1) *sizeof(char));
  sprintf(pidfile,"%s%s",home,PIDFILE);
  if ((fp = fopen(pidfile,"r")) != NULL) {
	fscanf(fp, "%d", (int *)&thepid);
	fclose(fp);
	free(home);
	free(pidfile);
  } else {
	free(home);
	free(pidfile);
  }
  return thepid;
}


/**********************************************************************************************/
